﻿using Microsoft.Extensions.Logging;
using SewingDressesBusinessLogic.MailWorker;
using SewingDressesContracts.BindingModels;
using SewingDressesContracts.BusinessLogicsContracts;
using SewingDressesContracts.SearchModels;
using SewingDressesContracts.StoragesContracts;
using SewingDressesContracts.ViewModels;
using SewingDressesDataModels.Enums;

namespace SewingDressesBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IClientStorage _clientStorage;
        private readonly AbstractMailWorker _mailLogic;
        static readonly object locker = new object();

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IClientStorage clientStorage, AbstractMailWorker worker)
        {
            _logger = logger;
            _orderStorage = orderStorage;
            _clientStorage = clientStorage;
            _mailLogic = worker;
        }

        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var elem = _orderStorage.GetElement(model);
            if (elem == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement found. Id:{Id}", elem.Id);
            return elem;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderId:{Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество заказов должно быть больше нуля");
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Цена заказа должна быть больше нуля");
            }
            _logger.LogInformation("Order. OrderId:{Id}. Sum:{Sum}", model.Id, model.Sum);
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                return false;
            }
            model.Status = OrderStatus.Принят;
            var order = _orderStorage.Insert(model);
            if (order == null) 
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            var orderView = _orderStorage.GetElement(new () { Id = order.Id });
            var clientView = _clientStorage.GetElement(new() { Id = orderView!.ClientId});

            SendEmail(clientView, orderView);
            return true;

        }

        private bool ChangeStatus(OrderBindingModel model, OrderStatus status) 
        {
            CheckModel (model, false);
            var element = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (element == null)
            {
                _logger.LogWarning("Find order failed");
                return false;
            }
            if (element.Status != status-1)
            {
                _logger.LogWarning("Status change failed");
                throw new InvalidOperationException("Невозможно перевести состояние заказа");
            }
            if (element.ImplementId != null)
                element.ImplementId = model.ImplementId;
            model.Status = status;
            if (model.Status == OrderStatus.Выдан) model.DateImplement = DateTime.Now;
            _orderStorage.Update(model);
			var orderView = _orderStorage.GetElement(new() { Id = model.Id });
			var clientView = _clientStorage.GetElement(new() { Id = orderView!.ClientId });

			SendEmail(clientView, orderView);
			return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            lock (locker)
            {
                return ChangeStatus(model, OrderStatus.Выполняется);
            }
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выдан);
        }
		public void SendEmail(ClientViewModel clientModel, OrderViewModel orderModel)
		{
			if (clientModel == null && orderModel == null)
			{
				return;
			}

			MailSendInfoBindingModel mailModel;

			if (orderModel.Status == OrderStatus.Принят)
			{
				mailModel = new MailSendInfoBindingModel
				{
					MailAddress = clientModel.Email,
					Subject = $"Order №{orderModel.Id}",
					Text = $"Your order №{orderModel.Id} by {orderModel.DateCreate} on {orderModel.Sum} was accepted!"
				};
			}
			else
			{
				mailModel = new MailSendInfoBindingModel
				{
					MailAddress = clientModel.Email,
					Subject = $"Order №{orderModel.Id}",
					Text = $"Order №{orderModel.Id} status was changed to {orderModel.Status}"
				};
			}


			_mailLogic.MailSendAsync(mailModel);
		}
	}
}
